import numpy as np
import pandas as pd

def shadow_augmentation(df, target_col, minority_class=1, total_synthetic=None,
                        light_dir=None, shadow_plane_normal=None, shadow_plane_point=None):

    features = [c for c in df.columns if c not in [target_col, "synthetic_id"]]
    minority_df = df[df[target_col] == minority_class]
    X = minority_df[features].values

    # Default light direction
    if light_dir is None:
        light_dir = np.array([1] + [0]*(len(features)-1))
    light_dir = light_dir / np.linalg.norm(light_dir)

    # Default plane normal = light direction
    if shadow_plane_normal is None:
        shadow_plane_normal = light_dir
    shadow_plane_normal = shadow_plane_normal / np.linalg.norm(shadow_plane_normal)

    # Default plane origin = 0 vector
    if shadow_plane_point is None:
        shadow_plane_point = np.zeros(len(features))

    synthetic_samples = []
    total_needed = total_synthetic or len(minority_df)

    def project_point(point):
        w = point - shadow_plane_point
        dist = np.dot(w, shadow_plane_normal)
        return point - dist * shadow_plane_normal

    for i in range(total_needed):
        idx = np.random.randint(0, len(minority_df))
        original = X[idx]

        shadow = project_point(original)
        scale = np.random.uniform(0.5, 1.5)
        synthetic = shadow_plane_point + (shadow - shadow_plane_point) * scale

        synthetic_samples.append(dict(
            {target_col: minority_class},
            **dict(zip(features, synthetic))
        ))

    return pd.DataFrame(synthetic_samples)



import pandas as pd
from .mirror import mirror_neighbor_aug_with_shape
from .shadow import shadow_augmentation

def hybrid_augmentation(df, target_col, minority_class=1, ratio=1.0):

    n = int(len(df[df[target_col] == minority_class]) * ratio)

    df_mirror = mirror_neighbor_aug_with_shape(df, target_col, minority_class, total_synthetic=n)
    df_shadow = shadow_augmentation(df, target_col, minority_class, total_synthetic=n)

    return pd.concat([df_mirror, df_shadow], ignore_index=True)



import numpy as np
import pandas as pd
import random
from sklearn.neighbors import NearestNeighbors

def mirror_neighbor_aug_with_shape(df, target_col, minority_class=1, total_synthetic=None,
                                   n_neighbors=5, shape_scale_range=(0.8, 1.2)):

    features = [c for c in df.columns if c not in [target_col, "synthetic_id"]]
    minority_df = df[df[target_col] == minority_class]
    X = minority_df[features].values

    nbrs = NearestNeighbors(n_neighbors=n_neighbors+1).fit(X)

    synthetic_samples = []
    total_needed = total_synthetic or len(minority_df)

    for i in range(total_needed):
        idx = random.randint(0, len(minority_df)-1)
        original = X[idx]

        _, indices = nbrs.kneighbors([original])
        neighbor = X[random.choice(indices[0][1:])]

        direction = neighbor - original
        direction *= np.random.uniform(shape_scale_range[0],
                                       shape_scale_range[1],
                                       size=len(direction))

        scale = np.random.uniform(0.6, 1.4)
        synthetic = original + scale * direction

        synthetic_samples.append(dict(
            {target_col: minority_class},
            **dict(zip(features, synthetic))
        ))

    return pd.DataFrame(synthetic_samples)



from .mirror import mirror_neighbor_aug_with_shape
from .shadow import shadow_augmentation
from .hybrid import hybrid_augmentation

__all__ = [
    "mirror_neighbor_aug_with_shape",
    "shadow_augmentation",
    "hybrid_augmentation"
]



from setuptools import setup, find_packages

setup(
    name="mh_sys_gen",
    version="0.1.1",
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    include_package_data=True,
    install_requires=[
        "numpy",
        "pandas",
        "scikit-learn",
    ],
)